import gtk
import gobject

from events import events
from event import event
from ev_editor import editor
from cal import cal
from docdb import docdb

from helpers import CalendarSelection

import Calendar

class events_list:
	def __init__(self, date, end, firstweekday=1):
		global _; _ = Calendar._
		self.gfx = Calendar.gfx
		
		self.firstweekday = firstweekday
		self.end = end
		self.date = date
		self.d = "%04d"%self.date[0] + "-" + "%02d"%self.date[1] + "-" + "%02d"%self.date[2]
		# create the interface:
		
		self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
		self.window.connect("delete_event", self.delete_event)
		self.window.connect("destroy", self.destroy)
		self.window.set_title(_("gdesklets calendar - Event List"))
		self.window.set_border_width(5)
		
		pixbuf = gtk.gdk.pixbuf_new_from_file(self.gfx + "calendar.png")
		self.window.set_icon(pixbuf)
		
		main_vbox = gtk.VBox(gtk.FALSE, 5)
		self.window.add(main_vbox)
		
		hbox = gtk.HBox(gtk.FALSE)
		label = gtk.Label(_("Your dates on"))
		hbox.pack_start(label, gtk.FALSE, gtk.FALSE, 5)
		self.calsel = CalendarSelection(self.d, self.firstweekday)
		hbox.pack_start(self.calsel, gtk.FALSE, gtk.FALSE)
		main_vbox.pack_start(hbox, gtk.FALSE, gtk.FALSE)
		
		self.calsel.entry.connect("activate", self.change_date)
		
		# adding the treeview
		hbox = gtk.HBox(gtk.FALSE, 5)
		
		sw = gtk.ScrolledWindow()
		sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
		self.model = gtk.TreeStore(gobject.TYPE_STRING, gobject.TYPE_STRING, 
		                      gobject.TYPE_PYOBJECT, gtk.gdk.Pixbuf)
		treeview = gtk.TreeView(self.model)
		treeview.set_size_request(300, 200)
		sw.add(treeview)
		hbox.pack_start(sw, gtk.FALSE, gtk.FALSE)
		main_vbox.pack_start(hbox, gtk.FALSE, gtk.FALSE)
		
		renderer = gtk.CellRendererPixbuf()
		column = gtk.TreeViewColumn('Icon', renderer, pixbuf=3)
		treeview.append_column(column)
		
		renderer = gtk.CellRendererText()
		column = gtk.TreeViewColumn(_("Time"), renderer, text=0)
		treeview.append_column(column)
		
		renderer = gtk.CellRendererText()
		column = gtk.TreeViewColumn(_("Summary"), renderer, text=1)
		treeview.append_column(column)
		
		selection = treeview.get_selection()
		selection.connect("changed", self.change_current_e, None)
		
		# adding the add, change and delete buttons
		vbox = gtk.VButtonBox()
		vbox.set_layout(gtk.BUTTONBOX_START)
		vbox.set_spacing(5)
		add = gtk.Button("Add", gtk.STOCK_ADD)
		change = gtk.Button("Change", gtk.STOCK_PROPERTIES)
		delete = gtk.Button("Delete", gtk.STOCK_DELETE)
		vbox.pack_start(add)
		vbox.pack_start(change)
		vbox.pack_start(delete)
		hbox.pack_start(vbox, gtk.FALSE, gtk.FALSE)
		
		add.connect("clicked", self.add_event, None)
		change.connect("clicked", self.edit_event, None)
		delete.connect("clicked", self.del_event, None)
		
		# the ok and close button
		hsep = gtk.HSeparator()
		main_vbox.pack_start(hsep)		


		buttons = gtk.HBox()
				
		#ok_button = gtk.Button("Ok", gtk.STOCK_OK)
		close_button = gtk.Button("Close", gtk.STOCK_CLOSE)
		buttons.pack_end(close_button, gtk.FALSE, gtk.FALSE, 5)
		#buttons.pack_end(ok_button, gtk.FALSE, gtk.FALSE, 5)
		buttons.show_all()
		main_vbox.pack_start(buttons, gtk.FALSE, gtk.FALSE, 3)
			
		close_button.connect("clicked", self.destroy, None)
		#ok_button.connect("clicked", self.save_and_quit, None)


		self.window.show_all()
		self.fill_model()


	def change_date(self, entry, data=None):
		d = self.calsel.get_date()

		self.date = int(d[0:4]), int(d[5:7]), int(d[8:10])
		self.d = d

		self.fill_model()
	
	def fill_model(self):

		if not hasattr(self, "model"):
			return 0
		self.model.clear()
		
		ev = events()
		evs = ev.get_events_from_date(self.date)
		types = []
		iters = {}
		
		for e in evs:
#			if types.count(e.get_type()) < 1:
#				iter = self.model.append(None, None)
#				self.model.set_value(iter, 0, e.get_type())
#				types.append(e.get_type())
#				iters[e.get_type()] = iter
			
			iter = self.model.append(None, None)
			if e.full_day == 1:
				self.model.set_value(iter, 0, _("Full Day"))
			else:
				self.model.set_value(iter, 0, e.start + " - " + e.end)
			self.model.set_value(iter, 1, e.summary)
			self.model.set_value(iter, 2, e.ID)
			
			if e.has_recurrence == 1:
				pixbuf = gtk.gdk.pixbuf_new_from_file(self.gfx + "recurrency.png")
			else:
				pixbuf = gtk.gdk.pixbuf_new_from_file(self.gfx + "event.png")
			
			p = pixbuf.scale_simple(18, 18, gtk.gdk.INTERP_BILINEAR)
			self.model.set_value(iter, 3, p)
	
	def add_event(self, widget, data=None):
		e = event()
		e.date = self.d
		ed = editor(e, self.fill_model, self.firstweekday)
	
	def change_current_e(self, selection, data=None):
		model, iter = selection.get_selected()
		test_iter = model.append(None, None)
		if type(iter) == type(test_iter):
			self.evID = model.get_value(iter, 2)
	
	def edit_event(self, widget, data=None):
		if not hasattr(self, "evID"):
			return
		doc = docdb()
		e = doc.get_event(self.evID)
		ed = editor(e, self.fill_model, self.firstweekday)
	
	def del_event(self, widget, data=None):
		if not hasattr(self, "evID"):
			return
		doc = docdb()
		doc.remove_event(self.evID)
		self.fill_model()
	
	def delete_event(self, widget, event, data=None):
		return gtk.FALSE
		
	def destroy(self, widget, data=None):
		self.window.destroy()
		self.end()
		
