from sensor.Sensor import Sensor
from utils.datatypes import *
from utils import i18n

import os
import commands
import time
import string
import xmms

# CornerXMMS sensor by Stefano Mozzi <stefano at mompracem dot org>
# Based on XMMS sensor by Jesse Andrews (C) 2003
# Under GPL (fsf.org)

class CornerXmmsSensor( Sensor ):

	def __init__( self ):
		global _; _ = i18n.Translator("cornerxmms-sensor")

		Sensor.__init__( self )

		self.__zoom = 1.0
		self.__counter = 0
		self.__change = 0
		self.__sign = 1
		
		self._add_timer( 0, self.__get_data )
		self._add_timer(30, self.__animation)

	def __get_data( self ):
		if (xmms.is_running()):
			cur = xmms.get_playlist_pos()
			title =	xmms.get_playlist_title( cur )

			bitrate, freq, nch = xmms.get_info()

			time = xmms.get_output_time() 
			self.__length =	xmms.get_playlist_time( cur )
			fill = float(time) / float(self.__length)

			if (self.__length == -1): 
				self.__length = 0

			isplaying =	xmms.is_playing()
			ispaused = xmms.is_paused()
			isplwin = xmms.is_pl_win()
			iseqwin = xmms.is_eq_win()
			ismainwin =	xmms.is_main_win()
			isrepeat = xmms.is_repeat()
			israndom = xmms.is_shuffle()

		else:
			title, self.__length, time, fill = "XMMS Not Loaded", 0, 0, 0
			bitrate, freq, nch = 0, 0, 0
			ispaused, isplaying = 0, 0
			isplwin, iseqwin, ismainwin = 0, 0, 0
			isrepeat, israndom = 0, 0

		titolo = self.__scrollo(title)
		chan = ['mono','stereo']

		output = self._new_output()

		output.set( "titolo",		titolo )
		output.set( "bitrate",		"%03d kbps  %02d khz  %s" % (bitrate/1000,freq/1000,chan[nch-1]))
		output.set( "length",		"%02d:%02d" % (self.__length/60000,(self.__length/1000)%60))
		output.set( "curtime",		"%02d:%02d" % (time/60000,(time/1000)%60))
		output.set( "fill",			int(fill*100) )
		output.set( "ispaused",		"gfx/pause%d.png" % (ispaused))
		output.set( "isplaying",	"gfx/play%d.png" % (isplaying))
		output.set( "isplwin",		"gfx/pl%d.png" % (isplwin))
		output.set( "iseqwin",		"gfx/eq%d.png" % (iseqwin))
		output.set( "ismainwin",	"gfx/ml%d.png" % (ismainwin))
		output.set( "isrepeat",		"gfx/rep%d.png" % (isrepeat))
		output.set( "israndom",		"gfx/rnd%d.png" % (israndom))

		self._add_timer( 1000, self.__get_data )
		self._send_output( output )

	def __scrollo(self,scritta):
		if (scritta == None):
			scritta = "No Song Loaded"
		scritta = "%s * " % scritta
		scritta = string.replace(scritta,"&","and")
		x = int(time.time())%len(scritta)
		titto = "%s%s%s" % (scritta[x:],scritta,scritta[:x])
		return titto

	def __hilight(self):
		self.__zoom = 1.0
		self.__sign = 1
		self.__change = 1
		self.__counter = 0

	def __unhilight(self):
		self.__zoom = 1.4
		self.__change = 1
		self.__counter = 0
		self.__sign = -1

	def __animation(self):
		if (self.__change):
			self.__change = 0
			if (self.__counter < 4):
				self.__zoom += 0.1 * self.__sign
				self.__counter += 1
				self.__change = 1
			
			output = self._new_output()
			output.set("icon-scale", self.__zoom)
			self._send_output(output)
		return 1

#	def get_configurator(self):
#		
#		configurator = self._new_configurator()
#		configurator.set_name(_("CornerXMMS"))
#		configurator.add_title(_("Settings"))
#		return configurator

	def call_action( self, action, path, args = []):
		
		# enter icon
		if (action == "enter"):
			self.__hilight()
		
		# leave icon panel
		elif (action == "leave"):
			self.__unhilight()
		
		elif (action == "moveto"):
			self.__px = args[0]
		
		elif (action == "jumpto"):
			tj = ( self.__length * self.__px ) / 160
			xmms.jump_to_time(tj)

		elif (action == "stop"):
			xmms.stop()

		elif (action == "play"):
			xmms.play()

		elif (action == "next"):
			xmms.playlist_next()

		elif (action == "prev"):
			xmms.playlist_prev()

		elif (action == "pause"):
			xmms.pause()

		elif (action == "eject"):
			xmms.eject()

		elif (action == "plwin"):
			xmms.pl_win_toggle(not(xmms.is_pl_win()))

		elif (action == "eqwin"):
			xmms.eq_win_toggle(not(xmms.is_eq_win()))

		elif (action == "mainwin"):
			xmms.main_win_toggle(not(xmms.is_main_win()))

		elif (action == "prefs"):
			xmms.show_prefs_box()

		elif (action == "reptog"):
			xmms.toggle_repeat()

		elif (action == "rndtog"):
			xmms.toggle_shuffle()

		elif (action == "quit"):
			if (xmms.is_running()):
				xmms.quit()
			else:
				os.system("cd ~ && xmms & disown")
				time.sleep(1)
				xmms.main_win_toggle(0)
#				xmms.play()

		elif (action == "turnin"):
			if (xmms.is_running()):
				xmms.enqueue_and_play(args[0])
			else:
				os.system("cd ~ && xmms & disown")
				time.sleep(1)
				xmms.main_win_toggle(0)
				xmms.enqueue_and_play(args[0])

def new_sensor( args ): return apply( CornerXmmsSensor, args )
