from sensor.Sensor import Sensor
from utils.datatypes import *
from utils import i18n

import time
import os
import commands

import libdesklets

def human_readable(value):
	if value >= (1024**4):
		return "%.2fT" % (value/(1024.0**4))
	if value >= (1024**3):
		return "%.2fG" % (value/(1024.0**3))
	if value >= (1024**2):
		return "%.2fM" % (value/(1024.0**2))
	if value >= 1000:
		return "%.2fK" % (value/1024.0)
	else : return "%i" % value

class Disk(Sensor):

	def __init__(self, default_mntpoint = "/", interval = "1000"):
		
		global _; _ = i18n.Translator("disk-sensor")
		
		self.__free = ""
		self.__used = ""
		self.__total = ""
		self.__percent = 0
		self.__targets = []
		Sensor.__init__(self)
		
		self._set_config_type("update", TYPE_INT, int(interval))
		self._set_config_type("target", TYPE_STRING, default_mntpoint)
		self._set_config_type("cut_mnt", TYPE_BOOL, 1)
		
		self._add_timer(int(interval), self.__on_tick)
		self._add_thread(self.__command_thread)
		
		
		list = libdesklets.disk.get_partitions()
		
		for targ in list :
			self.__targets.append([targ[0]+"("+targ[1]+")",targ[1]])

	def __on_tick(self):
		data = self._new_output()
		
		data.set("total", self.__total)
		data.set("used", self.__used)
		data.set("free", self.__free)
		data.set("percent", self.__percent)
		if self._get_config("cut_mnt"):
			cname = self._get_config("target").replace("/mnt/","")
			data.set("name",cname)
		else:
			data.set("name",self._get_config("target"))
		
		self._send_output(data)
		
		return 1

	def get_configurator(self):
		configurator = self._new_configurator()
		configurator.set_name(_("Disk"))
		configurator.add_title(_("Display Configuration"))
		configurator.add_spin(_("Update every (ms):"), "update",
					_("Updates the CPU load every x milliseconds"),
					100, 60000)
		configurator.add_option(_("Target:"), "target",
					_("Possible targets"),self.__targets)
		configurator.add_checkbox(_("Cut '/mnt/'"), "cut_mnt",
					_("Cuts the '/mnt/' in the name to reduce its size"))
		return configurator
	
	def __command_thread(self):
		while (1):
			if (self._is_stopped()): break
			
			size,used = libdesklets.disk.get_size(self._get_config("target"))

			self.__total = human_readable(size)
			self.__used = human_readable(used)
			self.__free = human_readable(size-used)
			self.__percent = int(100*used/size)
			
			time.sleep(self._get_config("update")*0.001)
				


def new_sensor(args): return apply(Disk, args)
