from sensor.Sensor import Sensor
from utils.datatypes import *
from utils import i18n

__author__ = "Bas van Oostveen"
__version__ = "0.1.0"
__license__ = "GPL"

#
# DisplayConstraints
# a small sensor which can bind a display to fixed bounds
#
class DisplayConstraints(Sensor):
	def __init__(self):
		global _; _ = i18n.Translator("DisplayConstraints")
		
		self.__active = 0
		self.__display_width = 100
		self.__display_height = 100
		
		Sensor.__init__(self)
		
		self._set_config_type("active", TYPE_BOOL, 0)
		self._set_config_type("display_width", TYPE_INT, self.__display_width)
		self._set_config_type("display_height", TYPE_INT, self.__display_height)
		self._set_config_type("always", TYPE_BOOL, 0)

		self._add_timer(500, self.__update)
	
	def __update(self):
		pchange = 0
		active = self._get_config("active")
		
		if (active==0 and self.__active==1):
			output =  self._new_output()
			output.set("width", -1)
			output.set("height", -1)
			self._send_output(output)
		elif (active==1 and self.__active==0):
			pchange = 1
		
		self.__active = active
		if (active==0): return 1	
		
		display_width = self._get_config("display_width")
		display_height = self._get_config("display_height")
		
		if (self._get_config("always")==1): pchange=1
		
		v1 = (display_width, display_height)
		v2 = (self.__display_width, self.__display_height)
		
		if (pchange == 1 or v1 != v2):
			output = self._new_output()
			
			if (pchange == 1 or display_width != self.__display_width):
				output.set("width", display_width)
				self.__display_width = display_width
			
			if (pchange == 1 or display_height != self.__display_height):
				output.set("height", display_height)
				self.__display_height = display_height
			
			self._send_output(output)
		
		return 1
		
	def get_configurator(self):
		configurator = self._new_configurator()
		configurator.set_name(_("DisplayConstraints"))
		configurator.add_title(_("DisplayConstraints version %s" % __version__))
		configurator.add_checkbox(_("Activate"), "active", _("Enabled or Disabled the DisplayConstraints."))
		configurator.add_checkbox(_("Always update"), "always", _("Always update the screen to configurated values.\nOtherwise only changes get updated."))
		configurator.add_title(_("Bounds:"))
		configurator.add_spin(_("Width:"), "display_width", _("Maximum width"), 0, 5000)
		configurator.add_spin(_("Height:"), "display_height", _("Maximum height"), 0, 5000)
		
		return configurator

def new_sensor(args): return apply(DisplayConstraints, args)
