from sensor.Sensor import Sensor
from utils.datatypes import *
from utils import i18n

import time
import os
import commands

class LTVCpu(Sensor):

	def __init__(self, count = "10", interval = "10", load = "white", user = "#E0E0E0", sys = "#CCCCCC", snap = "1"):
		
		global _; _ = i18n.Translator("cpu-sensor")
		
		self.__old = [ 0.0, 0.0, 0.0, 0.0 ]
		self.__load = []
		self.__snap = float(snap)
		Sensor.__init__(self)
		
		self._set_config_type("update", TYPE_INT, int(interval))
		self._set_config_type("count", TYPE_INT, int(count))

		self._set_config_type("load", TYPE_STRING, load)
		self._set_config_type("user", TYPE_STRING, user)
		self._set_config_type("sys", TYPE_STRING, sys)

		self._add_thread(self.__command_thread)
	def get_configurator(self):
		configurator = self._new_configurator()
		configurator.set_name(_("CPU Graph"))
		configurator.add_title(_("Graph Configuration"))
		configurator.add_spin(_("Samples:"), "count", _("Number of samples to display in the graph."), 2, 40)
		configurator.add_spin(_("Update every (ds):"), "update", _("Updates the graph every x deciseconds."), 1, 600)

	        configurator.add_color_selector(_("Graph Color:"), "load", _("Color used for the graph."))
#	        configurator.add_color_selector(_("Load:"), "load", _("Color used for the load graph."))
#	        configurator.add_color_selector(_("User:"), "user", _("Color used for the user graph."))
#	        configurator.add_color_selector(_("Sys:"), "sys", _("Color used for the sys graph."))
		return configurator
	def __draw_line(self, number, color):
		output = "color " + color + ", polygon "
		count = float( len( self.__load ) - 1 )
		step = 0 
		for i in self.__load :
			x = str( 2.0 * step / count - 1.0 )
			y = str( i[ number ] / 50.0 - 1.0 )
			if step == 0 :
				output = output + "-3 " + y + " "
			output = output + x + " " + y + " "
			if step == len( self.__load ) - 1 :
				output = output + "3 " + y + " 0 -3 0 "
			step = step + 1
		return output
	def __command_thread(self):
		while (1):
			if (self._is_stopped()): break
			
			count = self._get_config("count")
			while len( self.__load ) < count :
				self.__load = [[0,0,0]] + self.__load
			while len( self.__load ) > count :
				self.__load = self.__load[ 1: ]
			
			cpu = commands.getoutput("cat /proc/stat").splitlines()[0].split()
			
			uN = float(cpu[1])
			nN = float(cpu[2])
			sN = float(cpu[3])
			iN = float(cpu[4])
			
			u = uN - self.__old[0]
			n = nN - self.__old[1]
			s = sN - self.__old[2]
			i = iN - self.__old[3]
			
			t = u + n + s + i
			m = float(100) / t
			
			load = (u + n + s) * m
			user = (u + n) * m
			sys  = (s) * m
			
			self.__old = [ uN, nN, sN, iN ]
			self.__load = self.__load[ 1: ] + [[ load, user, sys ]]


			data = self._new_output()
			data.set("load", str(int(load))+"%")
			data.set("user", str(int(user))+"%")
			data.set("sys", str(int(sys))+"%")
			data.set("load_percent", str(int(load)))
			data.set("user_percent", str(int(user)))
			data.set("sys_percent", str(int(sys)))
			data.set("graph", "clear, " + self.__draw_line( 0, self._get_config( "load" ) ) )
#			data.set("graph", "clear, " + self.__draw_line( 0, self._get_config( "load" ) ) + "," + self.__draw_line( 1, self._get_config( "user" ) ) + "," + self.__draw_line( 2, self._get_config( "sys" ) ) )
			
			self._send_output(data)
			time.sleep(self._get_config("update")*0.1)
def new_sensor(args): return apply(LTVCpu, args)
