from sensor.Sensor import Sensor
from utils.datatypes import *
from utils import i18n

import time
import os
import commands

class LTVDisk(Sensor):

	def __init__(self, default_mntpoint = "/", default_icon = "hd", interval = "10", snap = "1"):
		
		global _; _ = i18n.Translator("disk-sensor")
		
		self.__free = ""
		self.__used = ""
		self.__total = ""
		self.__status = ""
		self.__percent = 0
		self.__percentsnap = 0
		self.__mounted = ""
		self.__targets = []
		self.__icons = [["Hard Disk","hd"],["Home","home"],["Directory","directory"],["Files","files"],["Floppy Disk","floppy"],["CD-ROM","cdrom"],["Network","network"]]
		self.__snap = float(snap)
		Sensor.__init__(self)
		
		self._set_config_type("update", TYPE_INT, int(interval))
		self._set_config_type("target", TYPE_STRING, default_mntpoint)
		self._set_config_type("icon", TYPE_STRING, default_icon)
		self._set_config_type("cut_mnt", TYPE_BOOL, 1)
		
		self._add_thread(self.__command_thread)
		
		data = commands.getoutput("cat /etc/fstab")
		data = data.splitlines()
		
		targid=0
		for line in data :
			fields = line.split()
			if ((len(line)>1) and (fields[0][:1]=='/') and (fields[1][:1]=='/')) :
				targ = fields[0].replace(":","")
				self.__targets.append([fields[0]+" : "+fields[1],fields[1]])
				targid += 1
			continue

	def get_configurator(self):
		configurator = self._new_configurator()
		configurator.set_name(_("Disk usage"))
		configurator.add_title(_("Display Configuration"))
		configurator.add_spin(_("Update every (ds):"), "update",
					_("Updates the disk usage every x deciseconds"),
					1, 600)
		configurator.add_option(_("Icon:"), "icon",
					_("Icon to use"),self.__icons)
		configurator.add_option(_("Target:"), "target",
					_("Possible targets"),self.__targets)
		configurator.add_checkbox(_("Remove '/mnt/'"), "cut_mnt",
					_("Removes the '/mnt/' from the beginning of names so it looks nicer."))
		return configurator
	
	def __command_thread(self):
		while (1):
			if (self._is_stopped()): break
			
			data = commands.getoutput("df -Ph")
			data = data.splitlines()
			success = 0;
			for line in data :
				fields = line.split()
				if fields[5] == (self._get_config("target")):
					self.__total = fields[1]
					self.__used = fields[2]
					self.__free = fields[3]
					perc = fields[4].replace("%","")
					self.__percent = int(perc)
					self.__percentsnap = self.__snap * int( float(perc) / self.__snap )
					self.__status = fields[4]
					self.__mounted = "mounted"
					success = 1
					break
			if success == 0 :
				self.__total = "0B"
				self.__used  = "0B"
				self.__free = "0B"
				self.__percent = 0
				self.__percentsnap = 0
				self.__status = "unmounted"
				self.__mounted = "unmounted"
				
			data = self._new_output()
		
			data.set("total", self.__total)
			data.set("used", self.__used)
			data.set("free", self.__free)
			data.set("percent", self.__percent)
			data.set("status", self.__status)
			data.set("mounted", self.__mounted)
			data.set("snap", self.__percentsnap)
			data.set("icon", "gfx/disk-" + self._get_config("icon") + ".png")
			if self._get_config("cut_mnt"):
				cname = self._get_config("target").replace("/mnt/","")
				data.set("name",cname)
			else:
				data.set("name",self._get_config("target"))
		
			self._send_output(data)
			
			time.sleep(self._get_config("update")*0.1)
				


def new_sensor(args): return apply(LTVDisk, args)
