# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

from sensor.Sensor import Sensor
from telnetlib import Telnet
from utils.datatypes import *
from utils import i18n

import time


class Mldonkey(Sensor):

	def __init__(self, interval = "5"):

		global _; _ = i18n.Translator("mldonkey-sensor")
		
		# call the constructor of the super class
		Sensor.__init__(self)
		
		self._set_config_type("update", TYPE_INT, int(interval))
		self._set_config_type("host", TYPE_STRING, "localhost")
		self._set_config_type("port", TYPE_STRING, "4000")
		self._set_config_type("login", TYPE_STRING, "admin")
		self._set_config_type("password", TYPE_SECRET_STRING, "")
		
		self._add_timer((int(interval)*1000), self.__on_tick)
		self._add_thread(self.__command_thread)
	
	def __on_tick(self):
		
		output = self._new_output()
		
		output.set("download", self.dl + " Kb/s ")
		output.set("upload", self.ul + " Kb/s ")
		output.set("version", self.version)
		output.set("dlfiles", self.dlfiles)
		
		self._send_output(output)
		
		return 1
	
	
	def get_configurator(self):
		configurator = self._new_configurator()
	
		configurator.set_name(_("Mldonkey info"))
		
		configurator.add_title(_("Server settings"))
		configurator.add_entry(_("Host:"),"host",_("host where the mldonkey core is located"))
		configurator.add_entry(_("Port:"),"port",_("mldonkey telnet port, default is 4000"))
		configurator.add_entry("Login:", "login","Login")
		configurator.add_entry("Password:", "password","Password",passwd = 1)
		
		configurator.add_title(_("Display"))
		configurator.add_spin(_("Update every (s):"), "update", _("Updates the display every x seconds"), 1, 6000)
		
		return configurator


	def __command_thread(self):
		
		while (1):
			if (self._is_stopped()): break
			
			try:
			
				tn = Telnet(self._get_config("host"), self._get_config("port"))
				tn.write('auth ' + self._get_config("login")  + ' ' + self._get_config("password") + '\r\n')
				tn.write('bw_stats\r\n')
				tn.write('version\r\n')
				tn.write('vd\r\n')
				tn.write('q\r\n')
				brut = tn.read_all()
				tn.close()
				data = brut.replace("\n"," ")
				data = data.split(" ")
				result = []
				
				if "login/password" in data:
					result = ["***","***","Bad login/password","***"]
				else:
					for field in data :
						if field == "Bad":
							result = ["***","***","Bad login/password","***"]
						else:
							if field == "\x1b[34mbw_stats\x1b[2;37;0m":
								result.append(data[data.index(field) + 2])
								result.append(data[data.index(field) + 11])
							elif field == "\x1b[34mversion\x1b[2;37;0m":	
								result.append(data[data.index(field) + 1] + " " + data[data.index(field) + 2])
							elif field == "\x1b[34mvd\x1b[2;37;0m":	
								result.append(data[data.index(field) + 2])
							
			except :
			
				result = ["***","***","Connection error","***"]
			
			self.dl = result[0]
			self.ul = result[1]
			self.version = result[2]
			self.dlfiles = result[3]
				
			
			
			time.sleep(self._get_config("update"))
	

def new_sensor(args): return apply(Mldonkey, args)
