from sensor.Sensor import Sensor
from utils.datatypes import *
from utils import i18n

import time
import types

has_wrapper=0
try:
	import textwrap
	has_wrapper=1
except:
	print "Textwrapping disabled, no python2.3 or no module textwrap"

from tailer import Tailer

__author__ = "Bas van Oostveen"
__version__ = "0.1.5"
__license__ = "GPL"

#
# Sensor for tailing multiple (log) files
# tails up to 5 log files in 1 display
#
class MultiTail(Sensor):

	def __init__(self, *args):
		global _; _ = i18n.Translator("MultiTail")
		self.text_buffer = []

		self.__border_left = 1
		
		Sensor.__init__(self)
		self._set_config_type("update", TYPE_INT, 15)
		self._set_config_type("border_left", TYPE_BOOL, self.__border_left)
		self._set_config_type("maxrow", TYPE_INT, 15)
		self._set_config_type("maxcol", TYPE_INT, 80)
		self._set_config_type("wrapprefix", TYPE_STRING, "")
		self._set_config_type("stlines", TYPE_INT, 2)
		self._set_config_type("logname", TYPE_BOOL, 1)
		self._set_config_type("rt1", TYPE_BOOL, 1)
		self._set_config_type("rt1_file", TYPE_STRING, "/var/log/messages")
		self._set_config_type("rt2", TYPE_BOOL, 1)
		self._set_config_type("rt2_file", TYPE_STRING, "/var/log/daemon.log")
		self._set_config_type("rt3", TYPE_BOOL, 0)
		self._set_config_type("rt3_file", TYPE_STRING, "")
		self._set_config_type("rt4", TYPE_BOOL, 0)
		self._set_config_type("rt4_file", TYPE_STRING, "")
		self._set_config_type("rt5", TYPE_BOOL, 0)
		self._set_config_type("rt5_file", TYPE_STRING, "")
		
		self._add_thread(self.__get_tail_thread)
		self._add_timer(2000, self.__update_display)
		
	def get_configurator(self):
		configurator = self._new_configurator()
		configurator.set_name(_("MultiTail"))
		configurator.add_title(_("MultiTail version %s" % __version__))
		configurator.add_title(_("Settings"))
		configurator.add_spin(_("Update every (s):"), "update",
							  _("Updates the display(s) every x seconds"),
							  1, 120)
		configurator.add_checkbox(_("Enable left border"), "border_left", _("Enable left border.\nWARNING if disabled and no lines are present display size could become 0x0 and transparent!"))
		configurator.add_checkbox(_("log names"), "logname", _("Show the name of the log before each line"))
		configurator.add_spin(_("Max lines:"), "maxrow", _("Maximum amount of lines (rows) printed."), 2, 50)
		if (has_wrapper==1):
			configurator.add_spin(_("Wrap at:"), "maxcol", _("Maximum amount of chars (columns) per line.\n0 disabled wrapping."), 0, 512)
			configurator.add_entry(_("Wrap prefix:"), "wrapprefix", _("Prepend this to each wrapped line."))
		else:
			configurator.add_title(_("Textwrapping is disabled."))
			configurator.add_title(_("either no python2.3 or no module textwrap found"))
		configurator.add_spin(_("Start lines:"), "stlines", _("Number of lines to print when a file is first read."), 0, 10)
		configurator.add_title(_("Tailers"))
		configurator.add_checkbox(_("Enabled"), "rt1", _("Enable Tailer #1"))
		configurator.add_file_selector(_("Log file #1:"), "rt1_file", _("The file which contents "
									   "is used to tail"))
		configurator.add_checkbox(_("Enabled"), "rt2", _("Enable Tailer #2"))
		configurator.add_file_selector(_("Log file #2:"), "rt2_file", _("The file which contents "
									   "is used to tail"))
		configurator.add_checkbox(_("Enabled"), "rt3", _("Enable Tailer #3"))
		configurator.add_file_selector(_("Log file #3:"), "rt3_file", _("The file which contents "
									   "is used to tail"))
		configurator.add_checkbox(_("Enabled"), "rt4", _("Enable Tailer #4"))
		configurator.add_file_selector(_("Log file #4:"), "rt4_file", _("The file which contents "
									   "is used to tail"))
		configurator.add_checkbox(_("Enabled"), "rt5", _("Enable Tailer #5"))
		configurator.add_file_selector(_("Log file #5:"), "rt5_file", _("The file which contents "
									   "is used to tail"))
		
		return configurator

	def __update_display(self):
		border_left = self._get_config("border_left")
		output = self._new_output()
		
		if (self.__border_left != border_left):
			output.set("border_left", border_left)
			self.__border_left = border_left
		else:
			output.set("border_left", self.__border_left)
		
		self._send_output(output)
		
		return 1
		
	def wrap(line, maxcol, wrapprefix):
		wrapped=[]
		if (len(line) > maxcol):
			pass
		else:
			wrapped.append("%s %s" % (wrapprefix, line))
		
		return wrapped
	
	def __get_tail_thread(self):
		rt1, rt2, rt3, rt4, rt5 = None, None, None, None, None
		rts = [['rt1', rt1], ['rt2', rt2], ['rt3', rt3],
		       ['rt4', rt4], ['rt5', rt5]]

		while (1):
			if (self._is_stopped()): break

			update = self._get_config("update")
			maxcol = self._get_config("maxcol")
			wrapprefix = self._get_config("wrapprefix")
			output = self._new_output()
			newlines = ""
			
			for rt in rts:
				if (self._get_config(rt[0])):
					fname = self._get_config(rt[0]+"_file")
					
					if (rt[1] is None):
						rt[1] = Tailer(self._get_config, fname)
					elif (rt[1].file != fname):
						rt[1].close()
						rt[1] = Tailer(self._get_config, fname)
					
					t=""
					for t in rt[1].tail():
						if (maxcol != 0 and has_wrapper):
							for w in textwrap.wrap(t, maxcol, subsequent_indent=wrapprefix):
								self.text_buffer.append('\n'+w.strip())
						else:
							self.text_buffer.append('\n'+t.strip())
					
					if (t[:7] == "[error]"):
						rt[1].close()
						rt[1] = None
						self._set_config(rt[0], 0)
					
				elif (rt[1] is not None):
					rt[1].close()
					rt[1] = None
			
			maxrow = self._get_config("maxrow")
			if (len(self.text_buffer) > maxrow):
				self.text_buffer = self.text_buffer[-maxrow:]
			
			if (self.text_buffer[0][0]=='\n'):
				self.text_buffer[0] = self.text_buffer[0][1:]

			output.set("logtail", ''.join(self.text_buffer))
			self._send_output(output)
			
			time.sleep(update)
	
def new_sensor(args): return apply(MultiTail, args)
