from sensor.Sensor import Sensor
from utils.datatypes import *
from utils import i18n

import time
import os

import libdesklets

def human_readable(value):
	if value >= (1024**4):
		return "%.2fT" % (value/(1024.0**4))
	if value >= (1024**3):
		return "%.2fG" % (value/(1024.0**3))
	if value >= (1024**2):
		return "%.2fM" % (value/(1024.0**2))
	if value >= 1000:
		return "%.2fK" % (value/1024.0)
	else : return "%i" % value

class Network(Sensor):

	def __init__(self, interface = "eth0", interval = "1000"):
		
		global _; _ = i18n.Translator("network-sensor")
		
		Sensor.__init__(self)
		
		self.__in = 0
		self.__out = 0
		self.__speed_in = 0
		self.__speed_out = 0
		self.__speed_in_max = 0
		self.__speed_out_max = 0
		self.__speed_in_percent = 0
		self.__speed_out_percent = 0
		self.__ip = ""
		self.__interface = interface
		self.__targets = []
		
		self._set_config_type("update", TYPE_INT, int(interval))
		self._set_config_type("target", TYPE_STRING, interface)
		self._set_config_type("format_in_out", TYPE_BOOL, 1)
		
		self._add_timer(int(interval), self.__on_tick)
		self._add_thread(self.__command_thread)
		
		for t in libdesklets.net.get_devices():
			self.__targets.append([t,t])
		

	def __on_tick(self):
		data = self._new_output()
		
		if self._get_config("format_in_out"):
			data.set("in", human_readable(self.__in)+"B")
			data.set("out", human_readable(self.__out)+"B")
		else:
			data.set("in", self.__in)
			data.set("out", self.__out)
			
		data.set("speed_in", human_readable(self.__speed_in)+"B/s")
		data.set("speed_out", human_readable(self.__speed_out)+"B/s")
		data.set("speed_in_max", human_readable(self.__speed_in_max)+"B/s")
		data.set("speed_out_max", human_readable(self.__speed_out_max)+"B/s")
		data.set("speed_in_percent", self.__speed_in_percent)
		data.set("speed_out_percent", self.__speed_out_percent)
		data.set("ip", self.__ip)
		data.set("interface", self._get_config("target"))

		self._send_output(data)
		
		return 1

	def get_configurator(self):
		configurator = self._new_configurator()
		configurator.set_name(_("Network"))
		configurator.add_title(_("Display Configuration"))
		configurator.add_spin(_("Update every (ms):"), "update",
					_("Updates the CPU load every x milliseconds"),
					100, 60000)
		configurator.add_option(_("Target:"), "target",
					_("Possible targets"),self.__targets)
		configurator.add_checkbox(_("Format in/out"), "format_in_out",
					_("Enables the in/out human readable format"))
		return configurator
	
	def __command_thread(self):
		while (1):
			if (self._is_stopped()): break
			
			if (self._get_config("target") != self.__interface):
				self.__speed_in_max=0
				self.__speed_out_max=0
			
			self.__interface = self._get_config("target")
			
			
			
			self.__ip = libdesklets.net.get_ipaddr(self.__interface)
			
			self.__in,self.__out = libdesklets.net.get_bytes_in_out(self.__interface)
			
			self.__speed_in,self.__speed_out = libdesklets.net.get_speed_in_out(self.__interface)
				
			if self.__speed_in_max < self.__speed_in : self.__speed_in_max = self.__speed_in
			if self.__speed_out_max < self.__speed_out : self.__speed_out_max = self.__speed_out

			if self.__speed_in_max!=0:
				self.__speed_in_percent = 100*self.__speed_in / self.__speed_in_max 
			else :
				self.__speed_in_percent = 0
				
			if self.__speed_out_max!=0:
				self.__speed_out_percent = 100*self.__speed_out / self.__speed_out_max
			else :
				self.__speed_out_percent = 0
			
			time.sleep(self._get_config("update")*0.001)

def new_sensor(args): return apply(Network, args)
