from sensor.Sensor import Sensor
from utils.datatypes import *
from utils import i18n

import re
import os
import time
import Queue
import commands

class PrayerTimeSensor(Sensor):
   PRAYER_SOURCE = "wget --quiet -O - " \
             "\"http://prayer.al-islam.com/country.asp?" \
             "Day=%(day)s&Month=%(month)s&Year=%(year)s&" \
             "Twilightflag=%(method)s&Asrway=%(asrway)s&%(dststr)s&" \
             "location=country&hiddcountry=%(country)s&hiddcity=%(city)s&" \
             "langm=eng&Prayer+Schedules=Prayer+Times\""

   REGGET = "\s*<TD ALIGN=\"CENTER\" nowrap>\s*(?P<stime>\S+)"
   REGSEP = "\s*\S*\s*"
   REGMID = "<TD ALIGN=\"CENTER\" nowrap>\s*"
   
   DSTARR = [ "time=Daily", "time=Daily&Dailysaving=Dailysaving" ]
   PRAYERNAMES = [ "fajr", "zuhr", "asr", "maghrib", "isha" ]
   
   nextcolor = "yellow"
   defcolor = "orange"
   update_frequency = 60
   prayertimes = []
   nextprayer = -1

   icons = [ "icons/fajr.png", "icons/zuhr.png", "icons/asr.png",
             "icons/maghrib.png", "icons/isha.png" ]

   def __init__(self):
      global _; _ = i18n.Translator("praytime-sensor")
      self.__queue = Queue.Queue(1)
      self.__path = os.getcwd()

      Sensor.__init__(self)
      self._set_config_type("city", TYPE_STRING, "Atlanta")
      self._set_config_type("country", TYPE_STRING, "GA")
      self._set_config_type("asrc", TYPE_STRING, "Shafi")
      self._set_config_type("method", TYPE_STRING, "IslamNorth")
      self._set_config_type("dstopt", TYPE_STRING, "auto")

      self._add_thread(self.__get_prayertimes_thread)
      self._add_timer(0, self.__get_prayertimes)


   def get_configurator(self):
      configurator = self._new_configurator()
      configurator.set_name(_("Prayer Times"))
      configurator.add_title(_("Settings"))
      configurator.add_entry(_("City:"), "city", _("Name of the City"))
      configurator.add_entry(_("Country/State:"), "country", 
                              _("Name of your Country or State" +
                                "(2 Letters only for States)"))
      
      configurator.add_title(_("Other Settings"))
      configurator.add_option(_("Method of Calculation:"), "method",
                              _("Method of PrayerTime Calculation"),
                              [(_("Umm Al-Qura University"), "UmULQura"),
                               (_("Muslim World League"), "IslamLeague"),
                               (_("Egyptian General Authority of Survey"),
                                  "Egyptian"),
                               (_("University of Islamic Sciences, Karachi"),
                                  "IslamScience"),
                               (_("Islamic Society of North America"),
                                  "IslamNorth")])

      configurator.add_option(_("Asr Calculation:"), "asrway",
                              _("Method of Asr Time Calculation"),
                              [(_("Hanafi"), "Hanafi"),
                               (_("Shafi'i"), "Shafi")])
      configurator.add_option(_("Daylight Savings:"), "dstopt",
                              _("Daylight Savings Time Option"),
                              [(_("Automatic"), "auto"),
                               (_("Daylight Savings"), "dston"),
                               (_("No Daylight Savings"), "dstoff")])

      return configurator


   def __calculateDistance(self):
      try:
         sclock = self.prayertimes
         active = self.nextprayer

         hrleft = (sclock[active])[3] - (time.localtime())[3]
         minleft = (sclock[active])[4] - (time.localtime())[4]
         if (hrleft < 0): hrleft = hrleft + 24
         if (minleft < 0):
            minleft = minleft + 60
            hrleft = hrleft - 1
         if (hrleft < 10): hrleft = '0' + (str)(hrleft)
         if (minleft < 10): minleft = '0' + (str)(minleft)
         return (str)(hrleft) + ":" + (str)(minleft)
      except:
         return "??:??"

   def __adjust_night_clock(self, clock):
      tstr = time.strftime('%H:%M', clock)
      strinfo = tstr.split(":")
      tmp = (int)(strinfo[0]) - 12
      tstr = (str)(tmp) + ":" + strinfo[1]
      c = time.strptime(tstr, '%H:%M')
      return time.strftime('%H:%M', c) + " pm"


   def __get_prayertimes(self):
      if (not self.__queue.empty()):
         output = self.__queue.get()
         self._send_output(output)
         
      self._add_timer(500, self.__get_prayertimes)

   def __get_prayertimes_thread(self):
      counter = 0
      curtime = int(time.time())
      old_values = ("", "", "", "", "")
      last_update = curtime - (self.update_frequency * 60) - 1

      while (1):
         if (self._is_stopped()): break
         city = self._get_config("city")
         country = self._get_config("country")
         method = self._get_config("method")
         asrway = self._get_config("asrway")
         dstopt = self._get_config("dstopt")
         
         if ((city, country, method, asrway, dstopt) != old_values
             or (last_update + (self.update_frequency * 60) <= curtime)):
            old_values = (city, country, method, asrway, dstopt)
            counter = 0

            curtime = time.localtime()
            day = curtime[2]
            month = curtime[1]-1
            year = curtime[0]

            if (dstopt == "auto"): tmp = time.daylight
            elif (dstopt == "dston"): tmp = 1
            else: tmp = 0

            dststr = self.DSTARR[tmp]
            curmin = curtime[4]
            curhour = curtime[3]
           
            fail, data = \
                  commands.getstatusoutput(self.PRAYER_SOURCE % vars())
            output = self._new_output()
            output.set("location", city + ", " + country)
            output.set("city", city)
            output.set("country", country)
            output.set("stime", _("Time:"))

            output.set("fajrstr", _("Fajr:"))
            output.set("shuruqstr", _("Shuruq:"))
            output.set("zuhrstr", _("Zuhr:"))
            output.set("asrstr", _("Asr:"))
            output.set("maghribstr", _("Maghrib:"))
            output.set("ishastr", _("Isha:"))
            output.set("titlestr", _("Prayer Times"))
            output.set("nextstr", _("Next Prayer:"))
  
            for i in range(0, 5):
               output.set(self.PRAYERNAMES[i]+"color", self.defcolor)

            try:
               stimes = [ '', '', '', '', '', ''] 
               for i in range(0, 6):
                  ctr = 0
                  regstr = ""
                  for j in range(0, i):
                     regstr = regstr + stimes[j] + self.REGSEP
                     if (j < (i-1)): regstr = regstr + self.REGMID
                  regstr = regstr + self.REGGET

                  regex = re.compile(regstr)
                  stimes[i] = regex.search(data).group("stime")

               output.set("shuruq", time.strftime('%H:%M',
                              time.strptime(stimes[1], '%H:%M')) + " am")
              
               for i in range(2, 6):
                  tmp = stimes[i].split(":")
                  if ((int)(tmp[0]) < 12):
                     hr = (str)((int)(tmp[0]) + 12)
                  stimes[i] = hr + ":" + tmp[1]
                  
               tmp = stimes[0:1]
               tmp[1:] = stimes[2:]
               stimes = tmp
               
               sclock = [ '', '', '', '', '' ]

               for i in range(0, 5):
                  sclock[i] = time.strptime(stimes[i], '%H:%M')

               active = 0
               for i in range(0, 4):
                  if ((sclock[i][3:5] < time.localtime()[3:5]) and
                        (sclock[i+1][3:5] > time.localtime()[3:5])):
                     active = i+1
                     break

               output.set(self.PRAYERNAMES[active]+"color", self.nextcolor)

               if (active == 0):
                  output.set("nextsalat", _("Fajr"))
               elif (active == 1):
                  output.set("nextsalat", _("Zuhr"))
               elif (active == 2):
                  output.set("nextsalat", _("Asr"))
               elif (active == 3):
                  output.set("nextsalat", _("Maghrib"))
               elif (active == 4):
                  output.set("nextsalat", _("Isha"))

               output.set("icon", os.path.join(self.__path,
                                               self.icons[active]))

               output.set("fajr", time.strftime('%H:%M', sclock[0]) + " am")

               for i in range(1, 5):
                  output.set(self.PRAYERNAMES[i],
                             self.__adjust_night_clock(sclock[i]))

               self.prayertimes = sclock
               self.nextprayer = active

               if (active > 0):
                  output.set("nextsalatime",
                             self.__adjust_night_clock(sclock[active]))
               else:
                  output.set("nextsalatime",
                             time.strftime('%H:%M', sclock[0]) + " am")

               tl = self.__calculateDistance()
               output.set("timeleft", tl)
               t = tl.split(":")
               output.set("timeleft_hm", t[0] + " hrs " + t[1] + " mins")

            except:
               output.set("shuruq", "??")
               for i in range(0, 5):
                  output.set(self.PRAYERNAMES[i], "??")
               output.set("timeleft", "??:??")
               output.set("timeleft_hm", "?? hrs ?? mins")
               output.set("nextsalat", "??")
               output.set("nextsalatime", "??:??")
               output.set("icon", os.path.join(self.__path,
                                               self.icons[1]))

            self.__queue.put(output)

         if ((counter % 60)==0):
            output.set("timeleft", self.__calculateDistance())
         counter += 1
         time.sleep(1)


print """
PrayerTime Sensor 0.1
http://www.prism.gatech.edu/~gte440v
Data obtained from http://prayer.al-islam.com
"""

def new_sensor(args): return apply(PrayerTimeSensor, args)
